//
//  MainVC.m
//  TOSR_BLE
//
//  Created by jun on 2017/6/8.
//  Copyright © 2017年 leekey. All rights reserved.
//

#import "MainVC.h"
#import "SerialGATT.h"
#import "LBMenuView.h"
#import "BLEViewController.h"
#import "ProgressHUD.h"

#import "NaviController.h"


#define BT_ASK_VERSION      200
#define BT_ASK_PASS         201
#define BT_ASK_WORKMODE     202
#define BT_ASK_STATUS       203

@interface MainVC ()<LBMenuViewDelegate,UITextFieldDelegate,BTSmartSensorDelegate,BTViewDeviceSetDelegate>
{
    BOOL isMemMode;
    BOOL isTouchFlag;
    BOOL statusBools[8];
    BOOL isVersionBackData;
    NSMutableArray *buttons;
}
@property (weak, nonatomic) IBOutlet UILabel *mStatus;

@property (weak, nonatomic) IBOutlet UIButton *btn_top;
@property (weak, nonatomic) IBOutlet UIButton *btn_left;
@property (weak, nonatomic) IBOutlet UIButton *btn_right;
@property (weak, nonatomic) IBOutlet UIButton *btn_bottom;
@property (weak, nonatomic) IBOutlet UISlider *seekBarA;
@property (weak, nonatomic) IBOutlet UISlider *seekBarB;
@property (weak, nonatomic) IBOutlet UIButton *btn_a_top;
@property (weak, nonatomic) IBOutlet UIButton *btn_a_bottom;
@property (weak, nonatomic) IBOutlet UIButton *btn_b_top;
@property (weak, nonatomic) IBOutlet UIButton *btn_b_bottom;

@property (weak, nonatomic) IBOutlet UIButton *btn_mode;

@property (strong, nonatomic) CBPeripheral *peripheral;
@property (strong, nonatomic) SerialGATT *sensor;
@property (strong, nonatomic) NSMutableArray *rssi_container;

@property(nonatomic,strong) NSArray *statusOnDatas;

@property(nonatomic,strong) LBMenuView *menuView;
@property(nonatomic,strong) UIButton *settingBG;


@property(nonatomic,retain) NSTimer *relayTimer;

@end

@implementation MainVC


- (void)viewDidLoad {
    [super viewDidLoad];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillResignActive:)
                                                 name:UIApplicationWillResignActiveNotification object:nil];
    
    
    _settingBG = [UIButton buttonWithType:UIButtonTypeCustom];
    _menuView = [LBMenuView menuView];
    _menuView.deletage = self;
    
    _statusOnDatas = @[@"6A",@"68",@"6B",@"69",@"64",@"66",@"65",@"67"];
    [self resetDefault];
}


//连接蓝牙
- (void)connectDevices{
    [self resetDefault];
    BLEViewController * bleViewController =[[BLEViewController alloc] init];
    bleViewController.delegate = self;
    NaviController *na = [[NaviController alloc]initWithRootViewController:bleViewController];
    [self presentViewController:na animated:YES completion:nil];
}


//选择蓝牙回调
- (void)didSelectDevice:(CBPeripheral*)per sensor:(SerialGATT*)sen{
    [self resetDefault];
    self.sensor = sen;
    self.sensor.delegate = self;
    [self.sensor connect:per];
    [ProgressHUD show:@"Connecting..."];
}


//连接成功回调
-(void)setConnect
{
    CFUUIDCreateString(kCFAllocatorDefault, (__bridge CFUUIDRef )self.sensor.activePeripheral.identifier);
    self.mStatus.text = self.sensor.activePeripheral.name;
    NSLog(@"=====连接成功===");
    [ProgressHUD show:@"Updating..."];
    [self askStatus];
}



-(void)askStatus{
    NSLog(@"=====请求Relay状态===");
    [ProgressHUD show];
    [self startRelayTimer];
}

-(void)startRelayTimer{
    if(self.relayTimer){
        [self.relayTimer invalidate];
    }
    self.relayTimer = [NSTimer scheduledTimerWithTimeInterval:0.3 target:self selector:@selector(sendStatus) userInfo:nil repeats:YES];
}

-(void)stopRelayTimer{
    if(self.relayTimer){
        [self.relayTimer invalidate];
    }
}

- (void)sendStatus{
    NSLog(@"=====请求状态间隔请求===");
    [self sendHexValue:@"43"];
}

//断开连接回调
-(void)setDisconnect
{
    [self resetDefault];
}


//收到蓝牙数据
-(void) serialGATTCharValueUpdated:(NSString *)UUID value:(NSData *)data
{
    unsigned long len = [data length];
    printf("\n\n=返回数据长度==len = %ld\n",len);
    Byte *resultByte = (Byte*)[data bytes];
    [ProgressHUD dismiss];
    if(len == 2){
        int mode = resultByte[0];
        isMemMode = mode == 2;
        NSString *imgStr = isMemMode?@"m_mode_momen":@"m_mode_latch";
        [self.btn_mode setBackgroundImage:[UIImage imageNamed:imgStr] forState:UIControlStateNormal];
        int status = resultByte[1];
        [self updateStatus:status];
    }
}


-(void)updateStatus:(int)status
{
    NSString *statusBir = [self toBinarySystemWithDecimalSystem:status];
    NSArray *statusDatas = [self reverseWordsInString:statusBir];
    for (int index = 0; index < 8; index++) {
        if(index < statusDatas.count){
            int va = [statusDatas[index] intValue];
            statusBools[index] = va ==1;
        }else{
            statusBools[index] = NO;
        }
    }
    [self.btn_left setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[0]?@"m_left_pressed":@"m_left_normal"]] forState:UIControlStateNormal];
    [self.btn_top setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[1]?@"m_top_pressed":@"m_top_normal"]] forState:UIControlStateNormal];
    [self.btn_right setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[2]?@"m_right_pressed":@"m_right_normal"]] forState:UIControlStateNormal];
    [self.btn_bottom setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[3]?@"m_bottom_pressed":@"m_bottom_normal"]] forState:UIControlStateNormal];
    
    [self.btn_a_top setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[4]?@"m_mo_top_pressed":@"m_mo_top_normal"]] forState:UIControlStateNormal];
    [self.btn_a_bottom setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[5]?@"m_mo_bottom_pressed":@"m_mo_bottom_normal"]] forState:UIControlStateNormal];
    [self.btn_b_top setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[6]?@"m_mo_top_pressed":@"m_mo_top_normal"]] forState:UIControlStateNormal];
    [self.btn_b_bottom setBackgroundImage:[UIImage imageNamed:[NSString stringWithFormat:@"%@",statusBools[7]?@"m_mo_bottom_pressed":@"m_mo_bottom_normal"]] forState:UIControlStateNormal];
}


//发送数据
-(void)sendHexValue:(NSString*)hexString
{
    int j=0;
    Byte bytes[hexString.length/2];
    for(int i=0;i<[hexString length];i++)
    {
        int int_ch;
        
        unichar hex_char1 = [hexString characterAtIndex:i];
        int int_ch1;
        if(hex_char1 >= '0' && hex_char1 <='9')
            int_ch1 = (hex_char1-48)*16;
        else if(hex_char1 >= 'A' && hex_char1 <='F')
            int_ch1 = (hex_char1-55)*16;
        else
            int_ch1 = (hex_char1-87)*16;
        i++;
        
        unichar hex_char2 = [hexString characterAtIndex:i];
        int int_ch2;
        if(hex_char2 >= '0' && hex_char2 <='9')
            int_ch2 = (hex_char2-48);
        else if(hex_char2 >= 'A' && hex_char2 <='F')
            int_ch2 = hex_char2-55;
        else
            int_ch2 = hex_char2-87;
        int_ch = int_ch1+int_ch2;
        bytes[j] = int_ch;
        j++;
    }
    NSData *newData = [[NSData alloc] initWithBytes:bytes length:hexString.length/2];
    NSLog(@"newData=%@",newData);
    [self.sensor write:self.sensor.activePeripheral data:newData];
}



- (IBAction)buttonUp:(id)sender {
    if (self.sensor.activePeripheral) {
        if(isTouchFlag && isMemMode){
            isTouchFlag = NO;
            [self sendHexValue:@"6C"];
        }
    }
}
-(void)sendRelayData:(NSDictionary*)dict
{
    while(isTouchFlag){
        if(isTouchFlag){
            [self sendHexValue:dict[@"data"]];
            [NSThread sleepForTimeInterval:0.3];
            
        }
    }
}

- (IBAction)buttonDown:(id)sender {
    if (self.sensor.activePeripheral) {
    
        int aValue =  (int)_seekBarA.value;
        int bValue = (int)_seekBarB.value;
        
        NSString *cmdA =  [NSString stringWithFormat:@"%@",[[NSString alloc] initWithFormat:@"%02x",aValue]];
        NSString *cmdB =[NSString stringWithFormat:@"%@",[[NSString alloc] initWithFormat:@"%02x",bValue]];
        NSString *data = @"";
        UIButton *button = (UIButton*)sender;
        NSLog(@"=pressed==>%ld",(long)button.tag);
        int index = (int)(button.tag - 100);
        switch (index) {
            case 0:
            case 1:
            case 2:
            case 3:
                data = [NSString stringWithFormat:@"%@%@%@%@",_statusOnDatas[index],cmdA,_statusOnDatas[index],cmdB];
                break;
            case 4:
            case 5:
                data = [NSString stringWithFormat:@"%@%@",_statusOnDatas[index],cmdA];
                break;
            case 6:
            case 7:
                data = [NSString stringWithFormat:@"%@%@",_statusOnDatas[index],cmdB];
                break;
            default:
                break;
        }
        if(isMemMode){
            isTouchFlag = YES;
            NSDictionary *dict = @{
                                   @"index":@(index),
                                   @"data":data
                                   };
            NSThread *thread = [[NSThread alloc] initWithTarget:self selector:@selector(sendRelayData:) object:dict];
            [thread start];
        }else{
            [self sendHexValue:[NSString stringWithFormat:@"%@",statusBools[index]?@"6C":data]];
        }
    }else{
        [self connectDevices];
    }
}

- (IBAction)onModeSwitch:(id)sender {
    if (self.sensor.activePeripheral) {
        isMemMode = !isMemMode;
        NSString *imgStr = isMemMode?@"m_mode_momen":@"m_mode_latch";
        [self.btn_mode setBackgroundImage:[UIImage imageNamed:imgStr] forState:UIControlStateNormal];
        [self sendHexValue:[NSString stringWithFormat:@"%@",isMemMode?@"42":@"41"]];
    }else{
        [self connectDevices];
    }
}


////////////////////底部菜单//////////////
-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    self.settingBG.frame = self.view.frame;
    self.settingBG.backgroundColor = [UIColor grayColor];
    self.settingBG.alpha = 0.0;
    [self.settingBG addTarget:self action:@selector(hideMenu:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.settingBG];
    
    self.menuView.frame = CGRectMake(0, self.view.height, self.menuView.width, self.menuView.height);
    [self.view addSubview:self.menuView];
    [self.view bringSubviewToFront:self.menuView];
    
    [UIView animateWithDuration:0.3 animations:^{
        self.settingBG.alpha = 0.2;
        self.menuView.frame = CGRectMake(0, self.view.height - self.menuView.height, self.menuView.width, self.menuView.height);
    } completion:^(BOOL finished) {
    }];
}

-(void)hideMenu:(UIButton*)btn
{
    [UIView animateWithDuration:0.3 animations:^{
        btn.alpha = 0;
        self.menuView.frame = CGRectMake(0, self.view.height, self.menuView.width, self.menuView.height);
    } completion:^(BOOL finished) {
        [self.menuView removeFromSuperview];
        [btn removeFromSuperview];
    }];
    
}
-(void)didSelectButton:(LBMenuView*)settingView menuType:(MENU_TYPE)menuType
{
    [self hideMenu:self.settingBG];
    if(menuType == MENU_TYPE_CONNECT){
        [self connectDevices];
    }else if(menuType == MENU_TYPE_DISCONNECT){
        [self resetDefault];
    }
}
////////////////////底部菜单结束//////////////


//恢复默认
-(void)resetDefault
{
    isTouchFlag = NO;
    for (int i = 0; i < 8; i++) {
        statusBools[i]= NO;
    }
    isMemMode = NO;
    [self updateStatus:0];
    _mStatus.text = @"Not Connect";
    if(self.relayTimer){
        [self.relayTimer invalidate];
        self.relayTimer = nil;
    }
    if ([self.sensor activePeripheral]) {
        if (self.sensor.activePeripheral.state == CBPeripheralStateConnected) {
            [self.sensor.manager cancelPeripheralConnection:self.sensor.activePeripheral];
            self.sensor.activePeripheral = nil;
        }
    }
}


////////////////其他信息///////////////
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.navigationController setNavigationBarHidden:YES animated:animated];
}

- (void)applicationWillResignActive:(NSNotification *)notification
{
    [self resetDefault];
}

-(void)viewDidUnload{
    [super viewDidUnload];
    [[NSNotificationCenter defaultCenter] removeObserver:UIApplicationWillResignActiveNotification];
}

-(UIStatusBarStyle)preferredStatusBarStyle
{
    return UIStatusBarStyleLightContent;
}


@end
